% this script runs through the hillshade of the safl flume, finds a local
% window, crops that window, does the identifygrains algorithm on that
% cropped area, uses analyzegrains to calculate D16 D50 and D84, and
% ultimately makes a matrix of [x y D16 D50 D84]. We do this twice, once
% with hminima turned on and once with it off.
clear all

flumeimagename = 'grid_18_42_hs1.tif';
A = imread(flumeimagename);
A = rgb2gray(A);

%limits of hillshade: upper left corner is [0 0] in matlab
%xmin should correspond to 18.000 m
%ymin should correspond to 0.190 m
%[18.000 0.190] should be lower left corner of image (this is how it plots
%out in arcgis and surfer)

xvec=[18:0.001:42];
yvec=[0.190:0.001:2.55];
[X,Y] = meshgrid(xvec,yvec);

%construct the moving window and run identifygrains in it
%center the moving window every 10 cm
%use 30 cm sides (center +/- 15cm)
xvec2 = [min(xvec):0.2:max(xvec)];
yvec2 = [min(yvec):0.2:max(yvec)];
halfwidth = 0.15;
totalcalcs = length(xvec2)*length(yvec2)
h=waitbar(0,'calculating');

jj=1;

for i=1:length(xvec2)
   for j=1:length(yvec2)
        xcenter = xvec2(i);
        ycenter = yvec2(j);
        xmin = xcenter - halfwidth;
        xmax = xcenter + halfwidth;
        ymin = ycenter - halfwidth;
        ymax = ycenter + halfwidth;
        
        %find indices in original image that fall within our window
        [indices_r, indices_c] = find(X >= xmin & X <= xmax & Y >= ymin ...
            & Y <= ymax);
        width = max(indices_c)-min(indices_c);
        height = max(indices_r)-min(indices_r);
        A2 = imcrop(A,[min(indices_c) min(indices_r) width height]);
        %figure, imshow(A2);
        hmin=0;
        grains = identifygrains2('grid_18_42_hs1.tif',A2,1,5,3,35,1,hmin,1,1,1,1,0);
        [D, cdf, D16, D50, D84] = analyzegrains(grains,2,2);
        
        summary(jj,1) = xcenter;
        summary(jj,2) = ycenter;
        summary(jj,3) = D16;
        summary(jj,4) = D50;
        summary(jj,5) = D84;
        waitbar(jj/totalcalcs);
        jj=jj+1;
       
   end
end

%%%%NOTE!!!
%%% DUE TO WEIRD MATLAB IMAGE REGISTRATION, THE Y VALUES FOR YCENTER ARE
%%% NOT THE SAME AS THE CART Y VALUES. TO TRANSFORM THESE INTO ACTUAL CART
%%% VALUES, USE THE FORMULA REALY = -YCENTER + 2.74. THIS MAY BE AVOIDABLE
%%% IF THE TIF IMAGE IS FLIPPED VERTICALLY BEFORE DOING THE MATLAB
%%% ANALYSIS, SO THAT THE MINIMUM Y VALUE IN CART COORDINATES CORRESPONDS
%%% TO THE TOP OF THE IMAGE.

close(h);

[XI,YI] = meshgrid(xvec2,yvec2);
ZI = griddata(summary(:,1), summary(:,2), summary(:,4),XI,YI);
vi = [min(summary(:,4)):0.5:max(summary(:,4))];
figure,
[C,h]=contour(XI,YI,ZI,vi);
clabel(C,h);
title('D50, hmin=1');

outputfilename = 'summary_hmin0.csv';
csvwrite(outputfilename,summary);

        