function [cdf2,D2] = convertgsd(cdf1,D1,exponent)

% [cdf2,D2] = convertgsd(cdf1,D1,exponent)
% 
% uses voidless cube model of Kellerhals and Bray to convert a cumulative
% grain size distribution from one type to another. The exponent determines
% the type of conversion performed. See Bunte and Abt p. 222 for a table of
% exponents corresponding to different types of conversions. Most common
% exponents are:
% volume by weight to area by number: -2; to area by weight: 1
% area by number to volume by weight: 2; to area by weight: 3
% area by weight to volume by weight: -1; to area by number: -3
% note that grid-by-number is equivalent to volume-by-weight
%
% D1 is vector of diameters (mm) of length N
% cdf1 is cdf from 0 to 100 corresponding to D1

%make a pdf from the cdf
D1copy = sort(D1,'ascend');
cdf1copy = sort(cdf1,'ascend');
D1next = D1copy;
D1next(1) = [];
D1copy(length(D1copy)) = [];
Dc = 2.^((log2(D1copy)+log2(D1next))./2); %vector of center of class (length N-1)

cdf1next = cdf1copy;
cdf1next(1) = [];
cdf1copy(length(cdf1copy)) = [];
pdf1 = cdf1next-cdf1copy;

factor = Dc.^exponent;
a = factor.*pdf1;
%adjust to 100%
pdfadj = a./sum(a).*100;
cdfadj = cumsum(pdfadj); %length N-1, needs a leading 0
cdf2 = zeros(length(cdfadj)+1,1);
cdf2(2:length(cdf2)) = cdfadj;
D2 = D1;
