%goal: divide up whole flume hillshade into overlapping cropped images. run
%identifygrains2 on those images, generate a long list of 'grains' (grain
%id, x, y, D). Then filter the grains so that grains whose centroids are
%within some threshold of each other are not double counted
%Oct 27 2008
%modified May 6 2010

%% go through the image and create grains matrices

clear all

flumeimagename = 'grid_18_42_hs1.tif';
A = imread(flumeimagename);
A = rgb2gray(A);

%limits of hillshade: upper left corner is [0 0] in matlab
%xmin should correspond to 18.000 m
%ymin should correspond to 0.190 m
%[18.000 0.190] should be lower left corner of image (this is how it plots
%out in arcgis and surfer)

xvec=[18:0.001:42];
yvec=[0.190:0.001:2.55];
[X,Y] = meshgrid(xvec,yvec);

%construct the moving window and run identifygrains in it
%center the moving window every 10 cm
% %use 30 cm sides (center +/- 15cm)
% xvec2 = [min(xvec):0.2:max(xvec)];
% yvec2 = [min(yvec):0.2:max(yvec)];
% halfwidth = 0.15;
% totalcalcs = length(xvec2)*length(yvec2)
% h=waitbar(0,'calculating
ywindowlength = 0.59;
xwindowlength = 0.60; %use a 60cm x 59cm window
nx = ceil((max(xvec)-min(xvec))/(xwindowlength/2));
ny = ceil((max(yvec)-min(yvec))/(ywindowlength/2));
totalcalcs = (nx-1)*(ny-1)
h=waitbar(0,'measuring grains');
grains1 = [];
grains0 = [];
jj=1;
for i=1:nx-1
    for j=1:ny-1
        xmin = min(xvec)+(i-1)*xwindowlength/2;
        xmax = xmin + xwindowlength;
        ymin = min(yvec)+(j-1)*ywindowlength/2;
        ymax = ymin + ywindowlength;
        
        %find indices in original image that fall within our window
        [indices_r, indices_c] = find(X >= xmin & X <= xmax & Y >= ymin ...
            & Y <= ymax);
        width = max(indices_c)-min(indices_c);
        height = max(indices_r)-min(indices_r);
        A2 = imcrop(A,[min(indices_c) min(indices_r) width height]);
        %figure, imshow(A2);
        %make grains1 and grains0 (grains matrices for hmin=1 and hmin=0)
        hmin=1;
        grains1temp = identifygrains2_ellipse('grid_18_42_hs1.tif',A,1,5,3,35,1,hmin,1,1,1,1,0);
        %convert x,y in grains to common coordinates (might need to change
        %y coordinates again later)
        grains1temp(:,2) = xmin+grains1temp(:,2)./1000;
        grains1temp(:,3) = ymin+grains1temp(:,3)./1000;
        grains1 = cat(1,grains1,grains1temp);
        clear grains1temp;
        hmin=0;
        grains0temp = identifygrains2_ellipse('grid_18_42_hs1.tif',A2,1,5,3,35,1,hmin,1,1,1,1,0);
        grains0temp(:,2) = xmin+grains0temp(:,2)./1000;
        grains0temp(:,3) = ymin+grains0temp(:,3)./1000;
        grains0 = cat(1,grains0,grains0temp);

        waitbar(jj/totalcalcs);
        disp(jj)
        jj=jj+1;
       
    end
end

close(h);

%% fix the grains for double counting

%now compute distance between each grain, get rid of ones within a
%threshold distance (b/c they are probably the same grain)
[r1,c1] = size(grains1);
[r0,c0] = size(grains0);
tooclose = 1e-4; %threshold (in m) for matching grain centers

%sort the whole thing first to increase likelihood of finding doubles -
%only adjacent points would be duplicated in this case
sumval1 = sum(grains1(:,2:6),2);
X1 = [grains1 sumval1];
X1 = sortrows(X1,[7 2 3]);
diff1 = diff(X1(:,2:7)); %could do cols 2:7 also
totdiff1 = sqrt(sum(diff1.^2,2));
badind1 = find(totdiff1 <= tooclose);
badind1 = ones(size(badind1)) + badind1;
grains1new = X1;
grains1new(badind1,:) = [];

X1 = sortrows(grains1,[2 3]);
X0 = sortrows(grains0,[2 3]);
% X1 = sortrows(grains1new,[4]);
% X0 = sortrows(grains0new,[4]);
diff1 = diff(X1(:,2:6));
diff0 = diff(X0(:,2:6));
totdiff1 = sqrt(sum(diff1.^2,2));
totdiff0 = sqrt(sum(diff0.^2,2));

badind1 = find(totdiff1 <= tooclose);
badind0 = find(totdiff0 <= tooclose);
badind1 = ones(size(badind1)) + badind1; %add 1 to index b/c these come from diff
badind0 = ones(size(badind0)) + badind0;

grains1new = X1;
grains1new(badind1,:) = [];
grains0new = X0;
grains0new(badind0,:) = [];

%%
% try using pdist
[r1,c1] = size(grains1);
[r0,c0] = size(grains0); 
maxnum = 5000;

grains1new = [];

for i=1:ceil(r1/maxnum)
    xmax = i*maxnum;
    xmin = (i-1)*maxnum+1;
    if r1<xmax; xmax = r1; end
    X1 = grains1(xmin:xmax,2:6);
%     X11 = X1(xmin:xmax,2:6);
    Y1 = pdist(X1);
    Y1 = squareform(Y1); %Y1(i,j) where i<j corresponds to dist between row i and j in X1
    [r,c]=find(Y1<2);
    inds = find(r<c);
    badinds = r(inds);
%     grainstemp = X1;
    X1(badinds,:) = [];
    grains1new = [grains1new; X1];
end

        