function grains = identifygrains2(imagename,A,pixpermm,diskradius,medfiltsize,...
    thresh1,thresh2,hminthresh,topedge,bottomedge,rightedge,leftedge,figureson)
%
% identifygrains2 - reads in image (A), rather than image file (imagename)
%
% grains = identifygrains2(imagename,A,pixpermm,diskradius,medfiltsize,...
%   thresh1,thresh2,hminthresh,topedge,bottomedge,rightedge,leftedge,figureson)
%
% 'edge' params: 1 to exclude, 0 to keep in during measurement
% figureson: 1 to generate figures along the way, 0 for no figures
% grains: [id x y D_mm]


%note: for uint8, white = 255 and black = 0
%       for binary, white = 1 and black = 0
% imagename = 's1_3151.tif';
% %parameters to control
% pixpermm = 1;
% diskradius = 5;
% medfiltsize = 3;
% thresh1 = 35;
% thresh2 = 1;
% hminthresh = 0;
% %exclude edges? 1 if yes, 0 if no
% topedge = 1;
% bottomedge = 1;
% rightedge = 1;
% leftedge = 1;

%read in the  and convert to grayscale
% A = imread(imagename);
% A = rgb2gray(A);
% if figureson == 1
%     figure, imshow(A); 
%     title(strcat(imagename,', step 1: grayscale image'),'Interpreter','none');
% end
    
%apply a median filter to blur grain imperfections, etc.
B = medfilt2(A, [medfiltsize medfiltsize]);
if figureson == 1
    figure, imshow(B); 
    title(strcat(imagename,', step 2: after median filter'),'Interpreter','none');
end

%define a structuring element to use to perform morphological transforms on
%the image in an attempt to improve contrast.
%here, C2 is probably what Graham et al use, C3 is based on an example in
%the matlab help file
SE = strel('disk', diskradius, 4);
C = imbothat(B, SE);
D = imtophat(B, SE);

C2 = imsubtract(B,C);
C3 = imsubtract(imadd(B,D),C);
if figureson == 1
    figure,imshow(C3); 
    title(strcat(imagename,', step3: after morphological transforms'),'Interpreter','none');
end

I = C3; %we will use I in the following code rather than C2, C3, etc

%now we need a histogram of intensity values, from which we can pick out
%the thresholds that will be used in the initial edge identification
[counts, intensity] = imhist(I); %counts = number of pix with intensity x
cumcounts = cumsum(counts);
cumpercent = cumcounts/max(cumcounts)*100;
%find lowindex (index in cumpercent vector that corresponds to the
%intensity for thresh1) and highindex (same thing for thresh2)
for i=1:length(cumpercent)-1
    if cumpercent(1) > thresh1
        lowindex = 1;
    else if cumpercent(i)<=thresh1 & cumpercent(i+1) > thresh1
            lowindex = i;
        end
    end
    if cumpercent(1) > thresh2
        highindex = 1;
    else if cumpercent(i) <= thresh2 & cumpercent(i+1) > thresh2
            highindex = i;
        end
    end
end

highbinary = (I<=intensity(highindex)); %binary map of darkest edges
lowbinary = (I<=intensity(lowindex));
%identify linked parts
[r,c] = find(highbinary>0);
E = bwselect(lowbinary,c,r,8); %E = binary edge map with edges =1
if figureson==1
    figure, imshow(~E); 
    title(strcat(imagename,', step 4: edge map after double threshold'),'Interpreter','none');
end

%perform watershed segmentation to split up connected areas in the new
%binary map E
disttransform = bwdist(E);
%figure, imshow(disttransform,[]);
disttransform = -disttransform;
%disttransform(E) = -Inf;
%do h-minima transform to suppress small pits in the EDM prior to watershed
%segmentation
disttransform = imhmin(disttransform,hminthresh);
% % note - hmin might work for photos, but for the dem hillshades it
% appears to be highly ineffective and most grains are severely
% undersegmented
F = watershed(disttransform);
%F is a label matrix. elements=0 are interstices, elements with numbers
%belong to the object (grain) identified by that number. max(F) should be
%the number of grains identified.
%replace grain id's on the edges with 0, if we are eliminating edges:
[r,c]=size(F);
for i=1:max(max(F))
    [r2,c2]=find(F==i);
    if topedge == 1
        zz = find(r2==1);
        if isempty(zz) == 0 % if object is at edge
            S=sparse(r2,c2,1,r,c); %makes sparse matrix with 1 at r2 and c2
            S=full(S); %convert to reg matrix
            S=logical(S); %convert to logical
            S=~S; %complement, now everything but elements of object i are 1
            F=S.*F; %should replace object i elements with 0
            continue
        end
    end
    if bottomedge == 1
        zz = find(r2==max(r));
        if isempty(zz) == 0 % if object is at edge
            S=sparse(r2,c2,1,r,c); %makes sparse matrix with 1 at r2 and c2
            S=full(S); %convert to reg matrix
            S=logical(S); %convert to logical
            S=~S; %complement, now everything but elements of object i are 1
            F=S.*F; %should replace object i elements with 0
            continue
        end
    end
    if leftedge == 1
        zz = find(c2==1);
        if isempty(zz) == 0 % if object is at edge
            S=sparse(r2,c2,1,r,c); %makes sparse matrix with 1 at r2 and c2
            S=full(S); %convert to reg matrix
            S=logical(S); %convert to logical
            S=~S; %complement, now everything but elements of object i are 1
            F=S.*F; %should replace object i elements with 0
            continue
        end
    end
    if rightedge == 1
        zz = find(c2==max(c));
        if isempty(zz) == 0 % if object is at edge
            S=sparse(r2,c2,1,r,c); %makes sparse matrix with 1 at r2 and c2
            S=full(S); %convert to reg matrix
            S=logical(S); %convert to logical
            S=~S; %complement, now everything but elements of object i are 1
            F=S.*F; %should replace object i elements with 0
            continue
        end
    end
end
%for some reason, watershed on its own gets rid of the 'thick' parts of the
%edge map after double threshold analysis, so all interstices are 1 pixel
%wide. The following lines of code are meant to combine the edges from the
%previous step with the watershed edges.
Fold = F; %preserve prior version, if wanted later
[m,n]=size(F);
[r,c]=find(E==1);
S=sparse(r,c,1,m,n); 
S=full(S);
S=logical(S);
S=~S; 
F=S.*F;

if figureson == 1
    figure, imshow(F); 
    title(strcat(imagename,', step 5: after watershed'),'Interpreter','none');
end

%make an overlay of the watershed on the origina b/w image
if figureson == 1
    out_red = A;
    out_green = A;
    out_blue = A;
    F2 = ~F;
    out_red(F2) = 255;
    out_green(F2) = 255;
    out_blue(F2) = 0;
    G = cat(3, out_red, out_green, out_blue);
    figure, imshow(G); 
    title(strcat(imagename,', watershed over original b/w image'),'Interpreter','none');
end

%measure grain sizes for objects identified in watershed segmentation
stats = regionprops(F, 'Centroid', 'MinorAxisLength');
%stats will have lots of grains with minoraxislength=0 and centroid [NaN
%NaN] if the edges are removed in the previous step.
%make a grains = [id x_centroid y_centroid MinorAxisLength] matrix
jj=1;
for i=1:length(stats)
    if stats(i).MinorAxisLength ~= 0
        grains(jj,1) = i;
        grains(jj,2:3) = stats(i).Centroid;
        grains(jj,4) = stats(i).MinorAxisLength/pixpermm;
        jj=jj+1;
    end
end