function [disttransform, E] = identifygrains2_ellipse_int(imagename,A,pixpermm,diskradius,medfiltsize,...
    thresh1,thresh2,hminthresh,topedge,bottomedge,rightedge,leftedge,figureson)
%
% identifygrains2 - reads in image (A), rather than image file (imagename)
% identifygrains2_ellipse = same as identifygrains2 but also outputs
% majoraxis and angle of ellipse. here, 
% grains: [id x y minoraxis(mm) majoraxis(mm) orientation(deg)]
%
% grains = identifygrains2(imagename,A,pixpermm,diskradius,medfiltsize,...
%   thresh1,thresh2,hminthresh,topedge,bottomedge,rightedge,leftedge,figureson)
%
% 'edge' params: 1 to exclude, 0 to keep in during measurement
% figureson: 1 to generate figures along the way, 0 for no figures
% grains: [id x y D_mm]


%note: for uint8, white = 255 and black = 0
%       for binary, white = 1 and black = 0
% imagename = 's1_3151.tif';
% %parameters to control
% pixpermm = 1;
% diskradius = 5;
% medfiltsize = 3;
% thresh1 = 35;
% thresh2 = 1;
% hminthresh = 0;
% %exclude edges? 1 if yes, 0 if no
% topedge = 1;
% bottomedge = 1;
% rightedge = 1;
% leftedge = 1;

%read in the  and convert to grayscale
% A = imread(imagename);
% A = rgb2gray(A);
% if figureson == 1
%     figure, imshow(A); 
%     title(strcat(imagename,', step 1: grayscale image'),'Interpreter','none');
% end
    
%apply a median filter to blur grain imperfections, etc.
B = medfilt2(A, [medfiltsize medfiltsize]);
if figureson == 1
    figure, imshow(B); 
    title(strcat(imagename,', step 2: after median filter'),'Interpreter','none');
end
fprintf(1,'finished median filter\n');

%define a structuring element to use to perform morphological transforms on
%the image in an attempt to improve contrast.
%here, C2 is probably what Graham et al use, C3 is based on an example in
%the matlab help file
SE = strel('disk', diskradius, 4);
C = imbothat(B, SE);
fprintf(1,'finished C\n');
D = imtophat(B, SE);
fprintf(1,'finished D\n');

% C2 = imsubtract(B,C);
C3 = imsubtract(imadd(B,D),C);
if figureson == 1
    figure,imshow(C3); 
    title(strcat(imagename,', step3: after morphological transforms'),'Interpreter','none');
end
fprintf(1,'finished C3 morphological transforms\n');

I = C3; %we will use I in the following code rather than C2, C3, etc

clear B C C2 C3 D

%now we need a histogram of intensity values, from which we can pick out
%the thresholds that will be used in the initial edge identification
[counts, intensity] = imhist(I); %counts = number of pix with intensity x
cumcounts = cumsum(counts);
cumpercent = cumcounts/max(cumcounts)*100;
%find lowindex (index in cumpercent vector that corresponds to the
%intensity for thresh1) and highindex (same thing for thresh2)
for i=1:length(cumpercent)-1
    if cumpercent(1) > thresh1
        lowindex = 1;
    else if cumpercent(i)<=thresh1 & cumpercent(i+1) > thresh1
            lowindex = i;
        end
    end
    if cumpercent(1) > thresh2
        highindex = 1;
    else if cumpercent(i) <= thresh2 & cumpercent(i+1) > thresh2
            highindex = i;
        end
    end
end

highbinary = (I<=intensity(highindex)); %binary map of darkest edges
lowbinary = (I<=intensity(lowindex));
%identify linked parts
[r,c] = find(highbinary>0);
E = bwselect(lowbinary,c,r,8); %E = binary edge map with edges =1
if figureson==1
    figure, imshow(~E); 
    title(strcat(imagename,', step 4: edge map after double threshold'),'Interpreter','none');
end
fprintf(1,'finished double thresholding (E)\n');

%perform watershed segmentation to split up connected areas in the new
%binary map E
disttransform = bwdist(E);
%figure, imshow(disttransform,[]);
disttransform = -disttransform;